RepTracker = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

local options = { 
	type='group',
	args = {
		track = {
		type = 'text',
		name = 'Current faction',
		desc = 'Sets which faction is tracked in this zone by default.  <none> to remove the default',
		get = "GetZone",
		set = "SetZone",
		usage = "<Your faction>",
			message = "%s tracked: %s",
		},
		sticky = {
			type = 'toggle',
			name = 'Toggle Sticky Rep',
			desc = 'Toggles whether the dynamic tracking changes are permanently stored as the default tracked faction',
			get = "GetSticky",
			set = "SetSticky"
		},
		dynamic = {
			type = 'toggle',
			name = 'Toggle Dynamic Rep Tracking',
			desc = 'Toggles whether the tracked rep changes to reflect the last faction you gained rep with',
			get = "GetDynamic",
			set = "SetDynamic"
		},
	},
}

RepTracker:RegisterDB("RepTrackerDB", "RepTrackerDBPC")

RepTracker:RegisterDefaults("profile", {
	ZoneToFaction = {},
	lastWatched = 1
} )

RepTracker:RegisterChatCommand({"/RepTracker", "/rt"}, options)

function RepTracker:OnEnable()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	if (self.db.profile.Dynamic == nil) then
		self.db.profile.Dynamic = true
	end
	if (self.db.profile.Dynamic == true) then
		self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
	end
end

function RepTracker:OnDisable()
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
	self:UnregisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
end

function RepTracker:ScanFactions()
	local factions = {}
	local name, isHeader, isCollapsed
	factions["none"] = 0
	for i = 0, GetNumFactions() do
		_, _, _, _, _, _, _, _, isHeader, isCollapsed = GetFactionInfo(i)
		if isHeader and isCollapsed then
			ExpandFactionHeader(i)
		end
	end
	for i = 0, GetNumFactions() do
		name, _, _, _, _, _, _, _, isHeader = GetFactionInfo(i)
		if not isHeader and name ~= nil then
			factions[string.lower(name)] = i
		end
	end
	self.db.profile.factions = factions
end

function RepTracker:ZONE_CHANGED_NEW_AREA()
	self:ScanFactions()
	local faction = self:GetZone()
	if faction == nil then
		self:Print("Zoned into "..GetZoneText()..", no faction tracked")
		self:Print("To set a faction to this zone type: /RepTracker track <faction>")
		SetWatchedFactionIndex(0)
	else
		faction = string.lower(faction)
		self:Print("Zoned into "..GetZoneText()..", tracking "..faction.." reputation")
		if(self.db.profile.factions[faction] ~= nil) then
			SetWatchedFactionIndex(self.db.profile.factions[faction])
		else
			self:Print("You have not made contact with that faction")
			SetWatchedFactionIndex(0)
		end
	end
end

function RepTracker:GetZone() return self.db.profile.ZoneToFaction[GetZoneText()] end

function RepTracker:GetDynamic() return self.db.profile.Dynamic end

function RepTracker:GetSticky() return self.db.profile.Sticky end

function RepTracker:SetDynamic()
	if(self.db.profile.Dynamic == nil) then
		self.db.profile.Dynamic = true
		self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
	else
		if(self.db.profile.Dynamic == false) then
			self.db.profile.Dynamic = true
			self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
		else
			self.db.profile.Dynamic = false
			self:UnregisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
		end
	end
end

function RepTracker:SetZone(value)
	if(self.db.profile.factions[string.lower(value)] ~= nil) then
		self.db.profile.ZoneToFaction[GetZoneText()] = string.lower(value)
		SetWatchedFactionIndex(self.db.profile.factions[string.lower(value)])
	else
		self:Print("You have not made contact with that faction")
	end
end

function RepTracker:SetSticky()
	if (self.db.profile.Sticky == true) then
		self.db.profile.Sticky = false
	else
		self.db.profile.Sticky = true
	end
end

function RepTracker:CHAT_MSG_COMBAT_FACTION_CHANGE()
	local faction
	faction,_ = RTGlobalParser_ParseMessage(arg1, FACTION_STANDING_INCREASED)
	if faction then
		self:changeBar(faction);
		return;
	end

	faction,_ = RTGlobalParser_ParseMessage(arg1, FACTION_STANDING_DECREASED)
	if faction then
		self:changeBar(faction);
		return;
	end

	faction,_ = RTGlobalParser_ParseMessage(arg1, FACTION_STANDING_CHANGED)
	if faction then
		self:changeBar(faction);
		return;
	end
end

function RepTracker:changeBar(faction)
	local oldName,_,_,_,_ = GetWatchedFactionInfo();
	for i=1, GetNumFactions() do
		local name,_,_,_,_,_,_,_,_,_ = GetFactionInfo(i);
		if ((name==faction) and not IsFactionInactive(i)) then
			if not(name==oldName) then
				SetWatchedFactionIndex(i);
				self:Print("Changing tracking to "..faction.." reputation")
				self.db.profile.lastWatched = i;
				if (self.db.profile.Sticky == true) then
					self:SetZone(faction)
				end
			end
		end
	end
end